import java.io.*;
import java.util.*;

public class ed2xml
{
  public static void main(String[] args)
  {
    if (args.length < 1)
    {
      System.err.println("Please specify an ED file.");
      System.exit(1);
    }

    try
    {
      HashMap dic = readDicFrom(args[0]);
      writeDicTo(dic, args[0] + ".xml");
    } catch (IOException ioe) {
      System.err.println("Conversion failed :(");
      System.exit(2);
    }
  }

  private static void writeDicTo(HashMap dic, String xmlFile) throws IOException
  {
    BufferedWriter out = new BufferedWriter(new FileWriter(xmlFile));

    String _out;

    _out = "<?xml version=\"1.0\" ?>";
    out.write(_out, 0, _out.length());
    out.newLine();
    out.newLine();

    _out = "<dictionnaire>";
    out.write(_out, 0, _out.length());
    out.newLine();

    ArrayList t;
    Object[] o = dic.keySet().toArray();
    String[] s = new String[o.length];
    for (int i = 0; i < s.length; i++)
      s[i] = o[i].toString();
    sortStrings(s);
    o = null;

    for (int i = 0; i < s.length; i++)
    {
      _out = "  <mot texte=\"" + s[i] + "\">";
      out.write(_out, 0, _out.length());
      out.newLine();

      t = (ArrayList) dic.get(s[i]);
      for (int j = 0; j < t.size(); j++)
      {
        _out = "    <traduction>" + t.get(j).toString() + "</traduction>";
        out.write(_out, 0, _out.length());
        out.newLine();
      }

      _out = "  </mot>";
      out.write(_out, 0, _out.length());
      out.newLine();
      out.newLine();

      out.flush();
    }

    _out = "</dictionnaire>";
    out.write(_out, 0, _out.length());
    out.newLine();

    out.flush();
    out.close();
  }

  private static HashMap readDicFrom(String edFile) throws IOException
  {
    BufferedReader in = new BufferedReader(new FileReader(edFile));

    ArrayList words;
    int pipeIndex = -1;
    StringTokenizer st;
    HashMap dic = new HashMap(1000);
    String line, entry, translations, token;

    while ((line = in.readLine()) != null)
    {
      pipeIndex = line.indexOf('|');
      if (pipeIndex == -1)
        continue;

      entry = line.substring(0, pipeIndex);
      if (dic.get(entry) == null)
      {
        words = new ArrayList();
        dic.put(entry, words);
      } else {
        words = (ArrayList) dic.get(entry);
      }

      translations = line.substring(pipeIndex + 1);
      st = new StringTokenizer(translations, ";");
      while (st.hasMoreTokens())
      {
        token = st.nextToken().trim();
        if (!words.contains(token))
          words.add(token);
      }
    }

    in.close();

    return dic;
  }

  public static void sortStrings(String[] strings)
  {
    sortStrings(strings, 0, strings.length -1);
  }

  private static void sortStrings(String[] a, int lo0, int hi0)
  {
    int lo = lo0;
    int hi = hi0;
    String mid;

    if (hi0 > lo0)
    {
      mid = a[(lo0 + hi0) / 2];

      while (lo <= hi)
      {
        while (lo < hi0 && a[lo].compareTo(mid) < 0)
          ++lo;

        while (hi > lo0 && a[hi].compareTo(mid) > 0)
          --hi;

        if (lo <= hi)
        {
          swap(a, lo, hi);
          ++lo;
          --hi;
        }
      }

      if (lo0 < hi)
        sortStrings(a, lo0, hi);

      if (lo < hi0)
        sortStrings(a, lo, hi0);
    }
  }

  private static void swap(String a[], int i, int j)
  {
    String T;
    T = a[i];
    a[i] = a[j];
    a[j] = T;
  }
}
