/*
 * 17:15:18 29/04/2001
 *
 * Ed.java - Embedded Dictionary
 * Copyright (C) 2000 Romain Guy
 * guy.romain@bigfoot.com
 * http://www.jext.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package ed;

// divers
import java.io.File;
import java.util.*;

// interface
import java.awt.*;
import java.awt.event.*;

// interface
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.filechooser.*;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
 * <strong>Ed</strong> signifie <strong>Embedded Dictionary</strong>.
 * Ed propose une interface simple pour accder rapidement  la
 * traduction d'un mot.<br>
 * Vous pouvez facilement ajouter Ed  votre logiciel:<p>
 * <pre>
 *   Ed ed = new Ed();
 *   myPanel.add(ed);
 *   ed.load(dicFile); // pour charger un dictionnaire  la main
 * </pre>
 * @author Romain Guy
 * @version 1.0
 */

public class Ed extends JPanel
{
  // membres privs
  private boolean triggered = true;
  private EdButton open;
  private JLabel statusLabel, inputFieldPrompt, resultsLabel;
  private JList resultsList;
  private JTextField inputField;
  // dcitionnaire charg
  private int entries;
  private EdDictionary dictionary;
  // le "seeker" attend avant de lancer la recherche
  private Seeker seeker;
  private DefaultListModel model;

  /**
   * L'objet Ed est un panneau affichant l'interface.
   */

  public Ed()
  {
    super(new BorderLayout());
    setBorder(new EmptyBorder(0, 2, 0, 2));

    // panneau du haut (bouton, champ de texte...)
    JPanel topPane = new JPanel(new BorderLayout());

    // nous ajoutons un titre
    JLabel titleLabel = new JLabel("Ed - Embedded Dictionary");
    titleLabel.setHorizontalAlignment(JLabel.LEFT);
    titleLabel.setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.black));
    topPane.add(titleLabel, BorderLayout.NORTH);

    // boutons
    ButtonsHandler buttonsHandler = new ButtonsHandler();
    open = new EdButton(getIcon("open.gif", Ed.class));
    // open.setMnemonic('O');
    open.setToolTipText("Ouvrir un dictionnaire");
    open.addActionListener(buttonsHandler);
    JToolBar toolbar = new JToolBar();
    toolbar.setFloatable(false);
    toolbar.add(open);

    // champ de texte
    inputField = new JTextField(8);
    inputField.addKeyListener(new KeyHandler());
    // libell du champ
    inputFieldPrompt = new JLabel(" Mot: ");
    JPanel promptPane = new JPanel(new FlowLayout(FlowLayout.LEFT));
    promptPane.add(inputFieldPrompt);
    promptPane.add(inputField);
    promptPane.add(toolbar);
    // JPanel leftedPane = new JPanel(new BorderLayout());
    // leftedPane.add(promptPane, BorderLayout.WEST);
    topPane.add(promptPane, BorderLayout.SOUTH);

    add(topPane, BorderLayout.NORTH);

    // liste de rsultats
    resultsList = new JList(model = new DefaultListModel());
    resultsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    // au sein d'un scroll pane, on pourra faire dfiler la liste
    JScrollPane resultsScroller = new JScrollPane(resultsList,
                                      JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                      JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    resultsLabel = new JLabel("Rsultats:");
    JPanel scrollerPane = new JPanel(new BorderLayout());
    scrollerPane.add(resultsLabel, BorderLayout.NORTH);
    scrollerPane.add(resultsScroller, BorderLayout.CENTER);
    add(scrollerPane, BorderLayout.CENTER);

    // statut
    statusLabel = new JLabel(" Bienvenue dans Ed !");
    statusLabel.setHorizontalAlignment(JLabel.RIGHT);
    add(statusLabel, BorderLayout.SOUTH);

    seeker = new Seeker();

    setMinimumSize(new Dimension(promptPane.getPreferredSize().width, getMinimumSize().height));
    setPreferredSize(getMinimumSize());
    triggerGUI(false);
  }

  // active/dsactive l'interface

  private void triggerGUI(boolean enabled)
  {
    inputFieldPrompt.setEnabled(enabled);
    inputField.setEnabled(enabled);
    resultsLabel.setEnabled(enabled);
    resultsList.setEnabled(enabled);
    triggered = enabled;
  }

  /**
   * Vrifie si Ed est prt  traduire.
   * @return true Si Ed est prt
   */

  public boolean isGUITriggered()
  {
    return triggered;
  }

  // change le message de statut

  private void setStatus(String status)
  {
    statusLabel.setText(status);
  }

  ////////////////////////////////////////////////////////////////////////////////////
  // MANIPULATION EXTERIEUR DE ED
  ////////////////////////////////////////////////////////////////////////////////////

  /**
   * Traduit un mot.
   */

  public void translate(String word)
  {
    if (isGUITriggered())
    {
      inputField.setText(word);
      setStatus(" " + loadMatches() + " / " + entries);
    }
  }

  /**
   * Renvoie la slection courante.
   */

   public String getSelectedTranslation()
   {
     return resultsList.getSelectedValue().toString();
   }

  ////////////////////////////////////////////////////////////////////////////////////
  // FIN DE L'API EXTERNE
  ////////////////////////////////////////////////////////////////////////////////////

  /**
   * Charge un nouveau dictionnaire.
   * @param dicFile Le chemin d'accs vers le dictionnaire
   * @see ed.EdFactory#buildFrom
   */

  public void load(String dicFile)
  {
    long ttim = System.currentTimeMillis();
    dictionary = EdFactory.buildFrom(dicFile);
    System.out.println("%% Ed: Chargement de " + dicFile + " en " +
                       ((double) (System.currentTimeMillis() - ttim) / 1000) +
                       " seconde(s)");

    if (dictionary == null)
    {
      triggerGUI(false);
      setStatus(" Bienvenue dans Ed !");
    } else {
      entries = dictionary.size();
      triggerGUI(true);
      setStatus(" " + entries + " entres.");
    }

    inputField.setText("");
    model.removeAllElements();
    resultsList.setModel(model);
  }

  // affiche la liste des rsultats

  private synchronized int loadMatches()
  {
    model.removeAllElements();

    String word = inputField.getText();
    if (word.length() == 0)
      return 0;

    EdEntry entry;
    int size = dictionary.size();

    for (int i = 0; i < size; i++)
    {
      entry = dictionary.getEntry(i);
      if (entry.word.startsWith(word))
        model.addElement(entry.toString());
    }

    resultsList.setModel(model);
    if (model.size() > 0)
      resultsList.setSelectedIndex(0);
    return model.size();
  }

   /**
   * Charge une image comme icone.
   * @param picture Le chemin de l'image
   * @param source La classe racine
   * @return Un <code>ImageIcon</code>
   */

  public static ImageIcon getIcon(String picture, Class source)
  {
    return new ImageIcon(Toolkit.getDefaultToolkit().getImage(
                         source.getResource(picture)));
  }

  // action ouvrir

  class ButtonsHandler implements ActionListener
  {
    public void actionPerformed(ActionEvent evt)
    {
      Object o = evt.getSource();
      if (o == open)
      {
        JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
        chooser.setDialogType(JFileChooser.OPEN_DIALOG);
        chooser.addChoosableFileFilter(new EdFileFilter());
        chooser.setDialogTitle("Ouvrir un dictionnaire...");
        if (chooser.showDialog(Ed.this, null) == JFileChooser.APPROVE_OPTION)
          load(chooser.getSelectedFile().getAbsolutePath());
      }
    }
  }

  // filtre des fichiers

  class EdFileFilter extends FileFilter
  {
    public boolean accept(File f)
    {
      if (f.isDirectory())
        return true;
      else
        return f.getAbsolutePath().endsWith(".ed.xml");
    }

    public String getDescription()
    {
      return "Dictionnaires Ed (*.ed.xml)";
    }
  }

  // reoit les vnements clavier du champ de texte

  class KeyHandler extends KeyAdapter
  {
    public void keyTyped(KeyEvent evt)
    {
      // test
      // System.out.println("%% evnment: " + evt.getKeyChar());
      seeker.seek();
    }
  }

  // la classe Seeker attend un petit moment aprs l'appui sur une touche
  // avant de charger les rsultats dans la liste

  class Seeker implements Runnable
  {
    private Thread t;

    /**
     * Attente de 300 milli-secondes avant de chercher les rsultats.
     */

    public void run()
    {
      try
      {
        t.sleep(300);
      } catch (InterruptedException ie) {
        return;
      }

      setStatus(" " + loadMatches() + " / " + entries);
    }

    /**
     * Dmarre le compteur.
     */

    public void seek()
    {
      if (t != null)
        takeSomeRest();
      t = new Thread(this);
      t.setName("Ed-Seeker");
      t.start();
    }

    /**
     * Arrte le compteur.
     */

    public void takeSomeRest()
    {
      t.interrupt();
      t = null;
    }
  }

  // point d'entre

  public static void main(String[] args)
  {
    MetalLookAndFeel.setCurrentTheme(new Theme());

    // nouvel objet Ed
    Ed ed = new Ed();

    // fentre
    final JFrame f = new JFrame("Ed - Dictionnaire en XML");
    f.getContentPane().add(ed);
    f.pack();
    f.setSize(320, 240);
    f.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent evt)
      {
        f.dispose();
        System.exit(0);
      }
    });
    f.setVisible(true);
  }
}

// End of Ed.java
