/*
 * 17:15:05 29/04/2001
 *
 * EdFactory.java - Embedded dictionary factory
 * Copyright (C) 2000 Romain Guy
 * guy.romain@bigfoot.com
 * http://www.jext.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package ed;

import java.io.*;
import java.util.ArrayList;

import com.microstar.xml.*;

/**
 * La classe <code>Edfactory</code> construit un dictionnaire
 *  partir d'un fichier XML.
 * @author Romain Guy
 * @version 1.0
 * @see ed.EdDictionary
 */

public final class EdFactory
{
  /**
   * <code>buildFrom(String)</code> charge un dictionnaire.
   * @param dicFile Le fichier contenant les donnes
   * @see ed.EdEntry ed.EdDictionary
   */

  public static final EdDictionary buildFrom(String dicFile)
  {
    // flux
    FileReader reader;
    // 'handler'
    EdHandler h;

    try
    {
      reader = new FileReader(dicFile);
      h = new EdHandler();

      // parseur XML
      XmlParser parser = new XmlParser();
      parser.setHandler(h);
      // lecture du fichier XML
      String file = new File(dicFile).getParent().replace(System.getProperty("file.separator").charAt(0), '/') + '/';
      if (file.charAt(0) != '/')
        file = "/" + file;
      parser.parse("file:" + file, null, reader);

    } catch (XmlException e) {
      System.err.println("Erreur de lecture de " + dicFile);
      System.err.println("A la ligne " + e.getLine() +
                         ", colonne " + e.getColumn());
      System.err.println(e.getMessage());

      return null;
    } catch (Exception e) {
      System.err.println("Lecture impossible: " + dicFile);
      return null;
    }

    try
    {
      // fermeture
      reader.close();
    } catch (IOException ioe) { }

    return h.getDictionary();
  }

  /**
   * Tri par ordre alphabtique.
   * @param string Les chanes  trier
   */

  public static void sortStrings(String[] strings)
  {
    sortStrings(strings, 0, strings.length -1);
  }

  private static void sortStrings(String[] a, int lo0, int hi0)
  {
    int lo = lo0;
    int hi = hi0;
    String mid;

    if (hi0 > lo0)
    {
      mid = a[(lo0 + hi0) / 2];

      while (lo <= hi)
      {
        while (lo < hi0 && a[lo].compareTo(mid) < 0)
          ++lo;

        while (hi > lo0 && a[hi].compareTo(mid) > 0)
          --hi;

        if (lo <= hi)
        {
          swap(a, lo, hi);
          ++lo;
          --hi;
        }
      }

      if (lo0 < hi)
        sortStrings(a, lo0, hi);

      if (lo < hi0)
        sortStrings(a, lo, hi0);
    }
  }

  /**
   * Echange deux lments du tableau.
   */

  private static void swap(String a[], int i, int j)
  {
    String T;
    T = a[i];
    a[i] = a[j];
    a[j] = T;
  }

  // la classe qui cre le dictionnaire

  static class EdHandler extends HandlerBase
  {
    // le dico
    private EdDictionary dictionary;
    // le mot, sa traduction
    private String word, trWord;
    // la liste des traductions trouves
    private ArrayList translations;

    public EdDictionary getDictionary()
    {
      return dictionary;
    }

    // dbut du document XML !

    public void startDocument()
    {
      // on cre le dico
      dictionary = new EdDictionary();
    }

    // lecture de donnes

    public void charData(char ch[], int start, int length)
    {
      // traduction trouve
      trWord = new String(ch, start, length);
    }

    // dbut de balise

    public void startElement(String elname)
    {
      // si la balise est <mot>
      if ("mot".equalsIgnoreCase(elname))
      {
        // on initialise la liste des traductions
        translations = new ArrayList();
      }
    }

    // fin de balise

    public void endElement(String elname)
    {
      // si la balise qui se finit est la balise <mot>
      if ("mot".equalsIgnoreCase(elname))
      {
        // on cre un tableau avec les traductions
        Object[] o = translations.toArray();
        // on transforme en tableau de chanes
        String[] s = new String[o.length];
        for (int i = 0; i < s.length; i++)
          s[i] = o[i].toString();

        // on le trie
        sortStrings(s);
        o = null;

        // on ajoute une nouvelle entre au dico
        dictionary.addEntry(new EdEntry(word, s));

        // si la balise est <traduction>
      } else if ("traduction".equalsIgnoreCase(elname)) {
        // on ajoute la traduction dans la liste
        translations.add(trWord);
      }
    }

    // attribut de balise

    public void attribute(String aname, String value, boolean isSpecified)
    {
      // l'attribut texte="" dfinit le mot  traduire
      if (aname.equalsIgnoreCase("texte"))
        word = value;
    }
  }
}

// End of EdFactory.java
